@@ -1,6 +1,13 @@
+"""
+Sumatra Server unit tests
 
+:copyright: Copyright 2010-2020 Andrew Davison
+:license: BSD 2-clause, see COPYING for details.
+"""
+
+from base64 import b64encode
 from django.test import TestCase
-from django.core.urlresolvers import reverse
+from django.urls import reverse
 from django.test.client import Client
 
 try:
@@ -9,6 +16,8 @@ except ImportError:
     import django.utils.simplejson as json
 import base64
 
+from sumatra_server.views import parse_accept_header
+
 
 OK = 200
 CREATED = 201
@@ -17,33 +26,18 @@ NOT_FOUND = 404
 NO_CONTENT = 204
 
 
-def deunicode(v):
-    """Convert unicode keys and values to str"""
-    if isinstance(v, dict):
-        new_v = {}
-        for kk, vv in v.items():
-            new_v[str(kk)] = deunicode(vv)
-    elif isinstance(v, unicode):
-        new_v = str(v)
-    elif isinstance(v, list):
-        new_v = [deunicode(d) for d in v]
-    else:
-        new_v = v
-    return new_v
-
-
 class BaseTestCase(TestCase):
-    fixtures = ['haggling', 'permissions']
+    fixtures = ["haggling", "permissions"]
 
     def setUp(self):
         # thanks to Thomas Pelletier for this function
         # http://thomas.pelletier.im/OK9/12/test-your-django-piston-api-with-auth/
         self.client = Client()
-        auth = '%s:%s' % ('testuser', 'abc123')
-        auth = 'Basic %s' % base64.encodestring(auth)
+        user_and_passwd = b64encode(b"%s:%s" % (b"testuser", b"abc123")).decode("ascii")
+        auth = "Basic %s" % user_and_passwd
         auth = auth.strip()
         self.extra = {
-            'HTTP_AUTHORIZATION': auth,
+            "HTTP_AUTHORIZATION": auth,
         }
 
     def assertMimeType(self, response, desired_mimetype):
@@ -52,29 +46,32 @@ class BaseTestCase(TestCase):
 
 
 class ProjectListHandlerTest(BaseTestCase):
-
     def test_GET_authenticated(self):
         prj_list_uri = reverse("sumatra-project-list")
         response = self.client.get(prj_list_uri, {}, **self.extra)
         self.failUnlessEqual(response.status_code, OK)
-        self.assertMimeType(response, "application/json")
+        self.assertMimeType(response, "application/vnd.sumatra.project-list-v4+json")
         data = json.loads(response.content)
         assert isinstance(data, list)
         self.assertEqual(len(data), 2)
         uris = {prj["uri"] for prj in data}
-        self.assertEqual(uris,
-                         {"http://testserver%sTestProject/" % prj_list_uri,
-                          "http://testserver%sTestProject2/" % prj_list_uri})
+        self.assertEqual(
+            uris,
+            {
+                "http://testserver%sTestProject/" % prj_list_uri,
+                "http://testserver%sTestProject2/" % prj_list_uri,
+            },
+        )
 
     def test_GET_anonymous(self):
         prj_list_uri = reverse("sumatra-project-list")
         response = self.client.get(prj_list_uri, {})
         self.failUnlessEqual(response.status_code, OK)
-        self.assertMimeType(response, "application/json")
+        self.assertMimeType(response, "application/vnd.sumatra.project-list-v4+json")
         data = json.loads(response.content)
+
         self.assertEqual(len(data), 1)
-        self.assertEqual(data[0]["uri"],
-                         "http://testserver%sTestProject2/" % prj_list_uri)
+        self.assertEqual(data[0]["uri"], "http://testserver%sTestProject2/" % prj_list_uri)
 
     def test_GET_format_html(self):
         self.client.login(username="testuser", password="abc123")
@@ -85,14 +82,11 @@ class ProjectListHandlerTest(BaseTestCase):
 
 
 class ProjectHandlerTest(BaseTestCase):
-
     def test_GET_private_authenticated(self):
-        prj_uri = reverse("sumatra-project",
-                          kwargs={"project": "TestProject"})
+        prj_uri = reverse("sumatra-project", kwargs={"project": "TestProject"})
         response = self.client.get(prj_uri, {}, **self.extra)
-        # print response.content
         self.assertEqual(response.status_code, OK)
-        self.assertMimeType(response, "application/json")
+        self.assertMimeType(response, "application/vnd.sumatra.project-v4+json")
         data = json.loads(response.content)
         assert isinstance(data, dict)
         self.assertEqual(data["id"], "TestProject")
@@ -101,15 +95,16 @@ class ProjectHandlerTest(BaseTestCase):
         assert isinstance(data["records"], list)
         # check we can also access all the records contained in this project
         for record in data["records"]:
-            touch = self.client.get(record, {}, **self.extra)  # should perhaps add support for HEAD to the server
+            touch = self.client.get(
+                record, {}, **self.extra
+            )  # should perhaps add support for HEAD to the server
             self.assertEqual(touch.status_code, OK)
 
     def test_GET_public_authenticated(self):
-        prj_uri = reverse("sumatra-project",
-                          kwargs={"project": "TestProject2"})
+        prj_uri = reverse("sumatra-project", kwargs={"project": "TestProject2"})
         response = self.client.get(prj_uri, {}, **self.extra)
         self.assertEqual(response.status_code, OK)
-        self.assertMimeType(response, "application/json")
+        self.assertMimeType(response, "application/vnd.sumatra.project-v4+json")
         data = json.loads(response.content)
         self.assertEqual(data["id"], "TestProject2")
         assert "anonymous" in data["access"]
@@ -119,55 +114,79 @@ class ProjectHandlerTest(BaseTestCase):
             self.assertEqual(touch.status_code, OK)
 
     def test_GET_public_anonymous(self):
-        prj_uri = reverse("sumatra-project",
-                          kwargs={"project": "TestProject2"})
+        prj_uri = reverse("sumatra-project", kwargs={"project": "TestProject2"})
         response = self.client.get(prj_uri, {})
         self.assertEqual(response.status_code, OK)
         data = json.loads(response.content)
         self.assertEqual(data["id"], "TestProject2")
-        assert "access" not in data
+        assert "access" not in data, str(data)
         # check we can also access all the records contained in this project
         for record in data["records"]:
             touch = self.client.get(record, {})
             self.assertEqual(touch.status_code, OK)
 
     def test_GET_private_anonymous(self):
-        prj_uri = reverse("sumatra-project",
-                          kwargs={"project": "TestProject"})
+        prj_uri = reverse("sumatra-project", kwargs={"project": "TestProject"})
         response = self.client.get(prj_uri, {})
         self.assertEqual(response.status_code, UNAUTHORIZED)
 
     def test_GET_nonexistent(self):
-        prj_uri = reverse("sumatra-project",
-                          kwargs={"project": "TestProject999"})
+        prj_uri = reverse("sumatra-project", kwargs={"project": "TestProject999"})
         response = self.client.get(prj_uri, {})
         self.assertEqual(response.status_code, NOT_FOUND)
 
+    def test_PUT_authenticated(self):
+        prj_uri = reverse("sumatra-project", kwargs={"project": "NewTestProject"})
+        response = self.client.put(prj_uri, {}, **self.extra)
+        self.assertEqual(response.status_code, CREATED)
+
 
 class RecordHandlerTest(BaseTestCase):
-
     def test_GET_no_data(self):
         label = "haggling"
-        rec_uri = reverse("sumatra-record",
-                          kwargs={"project": "TestProject",
-                                  "label": label})
+        rec_uri = reverse("sumatra-record", kwargs={"project": "TestProject", "label": label})
         response = self.client.get(rec_uri, {}, **self.extra)
-        # print response.content
         self.assertEqual(response.status_code, OK)
-        self.assertMimeType(response, "application/json")
+        self.assertMimeType(response, "application/vnd.sumatra.record-v4+json")
         data = json.loads(response.content)
         assert isinstance(data, dict)
         self.assertEqual(data["label"], label)
-        self.assertEqual(set(data.keys()),
-                         set(("executable", "label", "reason",
-                              "duration", "executable", "repository",
-                              "main_file", "version", "parameters",
-                              "launch_mode", "datastore", "outcome",
-                              "output_data", "timestamp", "tags", "diff",
-                              "user", "dependencies", "platforms",
-                              "project_id", "input_data", "input_datastore",
-                              "script_arguments", "stdout_stderr")))
-        #import pdb; pdb.set_trace()
+        from django.contrib.contenttypes.models import ContentType
+
+        self.assertEqual(ContentType.objects.filter(model="record")[0].pk, 18)
+        self.assertEqual(
+            set(data.keys()),
+            set(
+                (
+                    "executable",
+                    "label",
+                    "reason",
+                    "duration",
+                    "executable",
+                    "repository",
+                    "main_file",
+                    "version",
+                    "parameters",
+                    "launch_mode",
+                    "datastore",
+                    "outcome",
+                    "output_data",
+                    "timestamp",
+                    "tags",
+                    "diff",
+                    "user",
+                    "dependencies",
+                    "platforms",
+                    "project_id",
+                    "input_data",
+                    "input_datastore",
+                    "script_arguments",
+                    "stdout_stderr",
+                    "repeats",
+                )
+            ),
+        )
+        # import pdb; pdb.set_trace()
         self.assertEqual(data["tags"], ["foobar"])
         self.assertEqual(data["output_data"][0]["path"], "example2.dat")
         self.assertIsInstance(data["output_data"][0]["metadata"], dict)
@@ -176,27 +195,20 @@ class RecordHandlerTest(BaseTestCase):
         self.extra = {}  # use Django auth, not HTTP Basic
         self.client.login(username="testuser", password="abc123")
         label = "haggling"
-        rec_uri = reverse("sumatra-record",
-                          kwargs={"project": "TestProject",
-                                  "label": label})
+        rec_uri = reverse("sumatra-record", kwargs={"project": "TestProject", "label": label})
         response = self.client.get(rec_uri, {"format": "html"}, **self.extra)
         self.failUnlessEqual(response.status_code, OK)
         self.assertMimeType(response, "text/html")
 
     def test_GET_not_authenticated(self):
         label = "haggling"
-        rec_uri = reverse("sumatra-record",
-                          kwargs={"project": "TestProject",
-                                  "label": label})
+        rec_uri = reverse("sumatra-record", kwargs={"project": "TestProject", "label": label})
         response = self.client.get(rec_uri, {})
-        print response.content
         self.assertEqual(response.status_code, UNAUTHORIZED)
 
     def test_GET_nonexistent_record(self):
         label = "iquegfxnqiuehfiomehxgo"
-        rec_uri = reverse("sumatra-record",
-                          kwargs={"project": "TestProject",
-                                  "label": label})
+        rec_uri = reverse("sumatra-record", kwargs={"project": "TestProject", "label": label})
         response = self.client.get(rec_uri, {}, **self.extra)
         self.failUnlessEqual(response.status_code, NOT_FOUND)
 
@@ -204,9 +216,7 @@ class RecordHandlerTest(BaseTestCase):
         self.extra = {}  # use Django auth, not HTTP Basic
         self.client.login(username="testuser", password="abc123")
         label = "haggling"
-        rec_uri = reverse("sumatra-record",
-                          kwargs={"project": "TestProject",
-                                  "label": label})
+        rec_uri = reverse("sumatra-record", kwargs={"project": "TestProject", "label": label})
         response = self.client.get(rec_uri, {}, Accept="text/html", **self.extra)
         self.failUnlessEqual(response.status_code, OK)
         self.assertMimeType(response, "text/html")
@@ -220,101 +230,100 @@ class RecordHandlerTest(BaseTestCase):
                 "path": "lgljgkjhk",
                 "version": "iugnogn",
                 "name": "kljhnhn",
-                "options": "dfgdfg"
-            },
-            "repository": {
-                "url": "iuhnhc;<s",
-                "type": "iufvbfgbjlml",
-                "upstream": None
+                "options": "dfgdfg",
             },
+            "repository": {"url": "iuhnhc;<s", "type": "GitRepository", "upstream": None},
             "main_file": "OIYUGUIYFU",
             "version": "LUGNYGNYGu",
-            "parameters": {
-                "content": "oignuguygnug",
-                "type": "hjgjgn65878",
-            },
-            "input_data": [{
-                "path": "sfgshaeth",
-                "digest": "abcdef0123456789",
-                "metadata": {}
-            }],
+            "parameters": {"content": '{\n    "oignuguygnug": 3\n}', "type": "JSONParameterSet",},
+            "input_data": [
+                {
+                    "creation": None,
+                    "path": "sfgshaeth",
+                    "digest": "abcdef0123456789",
+                    "metadata": {},
+                }
+            ],
             "script_arguments": "p8yupyrprutot",
             "launch_mode": {
-                "type": "OIUNIU6nkjgbun",
-                "parameters": {},
+                "type": "SerialLaunchMode",
+                "parameters": {"options": None, "working_directory": "/path/to/wd"},
             },
             "datastore": {
-                "type": "mosigcqpoejf;",
-                "parameters": {
-                    "root": "/path/to/output/data"
-                }
+                "type": "FileSystemDataStore",
+                "parameters": {"root": "/path/to/output/data"},
             },
             "input_datastore": {
-                "type": "mosigcqpoejf;",
-                "parameters": {
-                    "root": "/path/to/input/data"
-                },
+                "type": "FileSystemDataStore",
+                "parameters": {"root": "/path/to/input/data"},
             },
             "outcome": "mihiuhpoip",
             "stdout_stderr": "erawoiawof23",
-            "output_data": [{
-                "path": "iugbufuyfiutyfitfy",
-                "digest": "0123456789abcdef",
-                "metadata": {}
-            }],
-            "timestamp": "2010-07-11T22:50:00",
+            "output_data": [
+                {
+                    "creation": None,
+                    "path": "iugbufuyfiutyfitfy",
+                    "digest": "0123456789abcdef",
+                    "metadata": {},
+                }
+            ],
+            "timestamp": "2010-07-11 22:50:00",
             "tags": ["abcd", "efgh", "ijklm", "tag with spaces"],
             "diff": "+++---",
             "user": "gnugynygy",
-            "dependencies": [{
-                "path": "moh,oh",
-                "version": "liuhiuhiu",
-                "name": "mohuuyfbn",
-                "module": "ouitfvbtfky",
-                "diff": "liugnig,lug",
-                "source": None,
-            }],
-            "platforms": [{
-                "system_name": "liugiuyhiuyg",
-                "ip_addr": "igng,iihih,i",
-                "architecture_bits": "vmsilughcqioej;",
-                "machine": "cligcnquefgx",
-                "architecture_linkage": "uygbytfkg",
-                "version": "luyhtdkguhl,h",
-                "release": "lufuytdydy",
-                "network_name": "ouifbf67",
-                "processor": "iugonuyginugugu"
-            }],
+            "dependencies": [
+                {
+                    "path": "moh,oh",
+                    "version": "liuhiuhiu",
+                    "name": "mohuuyfbn",
+                    "module": "python",
+                    "diff": "liugnig,lug",
+                    "source": None,
+                }
+            ],
+            "platforms": [
+                {
+                    "system_name": "liugiuyhiuyg",
+                    "ip_addr": "igng,iihih,i",
+                    "architecture_bits": "vmsilughcqioej;",
+                    "machine": "cligcnquefgx",
+                    "architecture_linkage": "uygbytfkg",
+                    "version": "luyhtdkguhl,h",
+                    "release": "lufuytdydy",
+                    "network_name": "ouifbf67",
+                    "processor": "iugonuyginugugu",
+                }
+            ],
+            "repeats": None,
         }
-        prj_uri = reverse("sumatra-project",
-                          kwargs={"project": "TestProject"})
+        prj_uri = reverse("sumatra-project", kwargs={"project": "TestProject"})
         rec_uri = "%s%s/" % (prj_uri, new_record["label"])
-        response = self.client.put(rec_uri, data=json.dumps(new_record),
-                                   content_type="application/json", **self.extra)
-        print response.content
+        response = self.client.put(
+            rec_uri,
+            data=json.dumps(new_record),
+            content_type="application/vnd.sumatra.record-v4+json",
+            **self.extra
+        )
         self.assertEqual(response.status_code, CREATED)
 
         response = self.client.get(rec_uri, {}, **self.extra)
         self.assertEqual(response.status_code, OK)
-        # pprint(deunicode(json.loads(response.content)))
         new_record.update(project_id="TestProject")
         self.maxDiff = None
-        #import pdb; pdb.set_trace()
-        self.assertEqual(new_record, deunicode(json.loads(response.content)))
+        self.assertEqual(new_record, json.loads(response.content))
 
     def test_PUT_existing_record_json(self):
-        prj_uri = reverse("sumatra-project",
-                          kwargs={"project": "TestProject"})
+        prj_uri = reverse("sumatra-project", kwargs={"project": "TestProject"})
         rec_uri = "%s%s/" % (prj_uri, "haggling")
         update = {
             "reason": "reason goes here",
             "outcome": "comments on outcome go here",
             "tags": ["tagA", "tagB", "tagC", "tag D"],
-            "version": "this should not be updated"
+            "version": "this should not be updated",
         }
-        response = self.client.put(rec_uri, data=json.dumps(update),
-                                   content_type="application/json", **self.extra)
-        print response.content
+        response = self.client.put(
+            rec_uri, data=json.dumps(update), content_type="application/json", **self.extra
+        )
         self.assertEqual(response.status_code, OK)
 
         response = self.client.get(rec_uri, {}, **self.extra)
@@ -329,9 +338,7 @@ class RecordHandlerTest(BaseTestCase):
 
     def test_DELETE_existing_record(self):
         label = "20111013-172503"
-        rec_uri = reverse("sumatra-record",
-                          kwargs={"project": "TestProject",
-                                  "label": label})
+        rec_uri = reverse("sumatra-record", kwargs={"project": "TestProject", "label": label})
         response = self.client.get(rec_uri, {}, **self.extra)
         self.assertEqual(response.status_code, OK)
 
@@ -343,8 +350,25 @@ class RecordHandlerTest(BaseTestCase):
 
     def test_DELETE_nonexistent_record(self):
         label = "mochirsueghcnisuercgheiosg"
-        rec_uri = reverse("sumatra-record",
-                          kwargs={"project": "TestProject",
-                                  "label": label})
+        rec_uri = reverse("sumatra-record", kwargs={"project": "TestProject", "label": label})
         response = self.client.delete(rec_uri, {}, **self.extra)
         self.assertEqual(response.status_code, NOT_FOUND)
+
+
+class UtilityFunctionTest(TestCase):
+    def test_parse_accept_header(self):
+        example_safari = "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8"
+        expected = ["text/html", "application/xhtml+xml", "application/xml", "*/*"]
+        self.assertEqual(parse_accept_header(example_safari), expected)
+
+        example_chrome = "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9"
+        expected = [
+            "text/html",
+            "image/webp",
+            "image/apng",
+            "application/xhtml+xml",
+            "application/signed-exchange",
+            "application/xml",
+            "*/*",
+        ]
+        self.assertEqual(parse_accept_header(example_chrome), expected)
